package ch.unige.solidify.test.util;

import static org.junit.jupiter.api.Assertions.assertEquals;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.Map;

import org.junit.After;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ErrorCollector;
import org.junit.runner.RunWith;
import org.springframework.core.io.ClassPathResource;
import org.springframework.test.context.junit4.SpringRunner;

import ch.unige.solidify.util.BagItTool;
import ch.unige.solidify.util.FileTool;

import gov.loc.repository.bagit.domain.Bag;

@RunWith(SpringRunner.class)
public class BagItTest {

  private final static String BAGIT_FOLDER = "bagit";
  private final static String BAG_FOLDER = "bag";
  @Rule
  public ErrorCollector collector = new ErrorCollector();

  @After
  public void purgeData() throws IOException {
    FileTool.deleteFolder(Paths.get(new ClassPathResource(BAGIT_FOLDER).getURI()).resolve(BAG_FOLDER));
  }

  @Test
  public void createTest() throws IOException {
    Path bagFolder = this.createBag();
    Bag bag = BagItTool.create(bagFolder, null);
    this.verifyBag(bag);
  }

  @Test
  public void readTest() throws IOException {
    Path bagFolder = this.createBag();
    BagItTool.create(bagFolder, this.getMetedata());
    Bag bag = BagItTool.read(bagFolder.toString());
    this.verifyBag(bag);
  }

  @Test
  public void checkTest() throws IOException {
    Path bagFolder = this.createBag();
    Bag bag = BagItTool.create(bagFolder, null);
    BagItTool.check(null, bag);
  }

  @Test
  public void quickCheckTest() throws IOException {
    Path bagFolder = this.createBag();
    Bag bag = BagItTool.create(bagFolder, null);
    BagItTool.quickCheck(null, bag);
  }

  private void verifyBag(Bag bag) {
    assertEquals("1.0", bag.getVersion().toString());
    assertEquals(2, BagItTool.getBagFileNumber(bag));
  }

  private Map<String, String> getMetedata() {
    HashMap<String, String> metadata = new HashMap<>();
    return metadata;
  }

  private Path createBag() throws IOException {
    Path source = Paths.get(new ClassPathResource(BAGIT_FOLDER).getURI());
    Path destination = source.resolve(BAG_FOLDER);
    for (Path f : FileTool.findFiles(source)) {
      FileTool.copyFile(f, destination.resolve(f.getFileName()));
    }
    return destination;
  }

}
